﻿using System;
using System.Collections.ObjectModel;
using ThinkGeo.MapSuite.Core;

namespace CacheGenerator
{
    internal class LayerProvider
    {
        // You can insert your layer to "layersToCache" in this function, they will be used for cache.
        public static Collection<Layer> GetLayersToCache()
        {
            //Collection<Layer> layersToCache = new Collection<Layer>();

            //ShapeFileFeatureLayer layer = new ShapeFileFeatureLayer(@"D:\Kevin-ThinkGeo\MapSuite3.0\Development\DesktopEdition\HowDoISamples\SampleData\Data\Countries02.shp", ShapeFileReadWriteMode.ReadOnly);

            //layer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(100, GeoColor.SimpleColors.LightBlue), GeoColor.SimpleColors.LightBlue, 1);
            //layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            //layersToCache.Add(layer);

            var layersToCache = GetLayersToCache(@"..\..\App_Data\HTSL_1.jp2");
            return layersToCache;
        }

        private static double GetConfiguredZoomScale(double maxZoomScale, int configuredZoomLevelID)
        {
            double factorToDevide = Math.Pow(2, (configuredZoomLevelID));
            double configuredZoomScale = maxZoomScale / factorToDevide;
            return configuredZoomScale;
        }

        private static Collection<double> CreateCustomScale(int configuredZoomLevelID)
        {
            Collection<double> scalesToCache = new Collection<double>();
            const int totalNumberOfZoomLevelsTobeConfigured = 20 - 1;//Configured value starts from 0 till 19.Hence the constant is 19
            const string functionName = "CreateZoomLevels";
            try
            {
                ZoomLevelSet zoomLevelSet = new ZoomLevelSet();
                var upperZoomScale = zoomLevelSet.ZoomLevel20.Scale;
                var lowerZoomScale = 591658710.9;
                var configuredZoomScale = GetConfiguredZoomScale(lowerZoomScale, configuredZoomLevelID);

                var intialZoomScale = configuredZoomScale;
                int stepsNeededinBetweenScales = 0;
                double stepScale = 0;
                var availableZoomLevels = totalNumberOfZoomLevelsTobeConfigured - configuredZoomLevelID;
                //This many number of zoom levels are available for us from configuration
                if (configuredZoomLevelID != 0) //Recreate zoom levels
                {
                    if (availableZoomLevels != 0)
                    {
                        if (availableZoomLevels != 1)
                            stepsNeededinBetweenScales = (totalNumberOfZoomLevelsTobeConfigured - availableZoomLevels) /
                                                         (availableZoomLevels - 1);
                        //Only available availableZoomLevels - 1 levels
                        //are there for inserting step zoom level
                        else
                            stepsNeededinBetweenScales = totalNumberOfZoomLevelsTobeConfigured - 1;
                        // In this case we have only one step to insert all zoom levels.
                    }
                    scalesToCache.Add(configuredZoomScale);
                    for (int majorZoomLevel = 1; majorZoomLevel < availableZoomLevels; majorZoomLevel++)
                    {
                        for (int step = 1; step <= stepsNeededinBetweenScales; step++)
                        {
                            var lowerStepScale = intialZoomScale;
                            var upperStepScale = intialZoomScale / 2;
                            stepScale = (lowerStepScale - upperStepScale) / (stepsNeededinBetweenScales + 1);
                            // Since we are adding major zoom levels in upper loop
                            // adding 1 to stepsNeededinBetweenScales for calculation
                            if (scalesToCache.Count >= totalNumberOfZoomLevelsTobeConfigured - 1)
                                break; // If at all the count  goes more than 18 due to round off break the loop
                            scalesToCache.Add(lowerStepScale - stepScale * step);
                        }
                        intialZoomScale = intialZoomScale / 2;
                        scalesToCache.Add(intialZoomScale);
                    }
                    //Due to round off while calculting the number of stepsneeded inbetween, we will loose some zoom levels or get some zoom levels
                    if (scalesToCache.Count < totalNumberOfZoomLevelsTobeConfigured)
                    {
                        var deficitLevels = totalNumberOfZoomLevelsTobeConfigured - scalesToCache.Count;
                        var scaleToBeadded = (intialZoomScale - upperZoomScale) / (deficitLevels + 1);
                        // adding one to ensure that we can yet have one more last zoom level to be added
                        //We need to increase some steps because of round off while calculating number of steps needed in between
                        for (int step = 1; step <= deficitLevels; step++)
                            scalesToCache.Add((intialZoomScale - step * scaleToBeadded));
                    }
                    scalesToCache.Add(upperZoomScale);// This is the last scale for the map
                }
                else //If map is configured with 0 zoom level,add only default zoom levels
                {
                    for (int majorZoomLevel = configuredZoomLevelID;
                         majorZoomLevel <= totalNumberOfZoomLevelsTobeConfigured;
                         majorZoomLevel++)
                    {
                        intialZoomScale = intialZoomScale / 2;
                        scalesToCache.Add(intialZoomScale);
                    }
                }
                return scalesToCache;
            }
            catch (Exception ex)
            {
                throw;
            }
        }

        // You can insert your scales to "scalesToCache" in this function, they will be used for cache.
        public static Collection<double> GetScalesToCache()
        {
            return CreateCustomScale(0);
        }

        private static string _layerName;

        public static string LayerName
        {
            get { return _layerName; }
            set { _layerName = value; }
        }

        internal static Collection<Layer> GetLayersToCache(string p)
        {
            Collection<Layer> layersToCache = new Collection<Layer>();
            LayerName = p;
            if (p.Contains("ecw"))
            {
                var layertoadd = new EcwRasterLayer(p);
                layersToCache.Add(layertoadd);
            }
            else if (p.Contains("jp2"))
            {
                var layertoadd = new Jpeg2000RasterLayer(p);
                layersToCache.Add(layertoadd);
            }

            return layersToCache;
        }
    }
}