﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace WPFDisplayASimpleMap
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            wpfMap1.MapUnit = GeographyUnit.DecimalDegree;
            wpfMap1.CurrentExtent = new RectangleShape(-155.733, 95.60, 104.42, -81.9);
            WorldMapKitWmsWpfOverlay worldOverlay = new WorldMapKitWmsWpfOverlay();
            //wpfMap1.Overlays.Add("WMK", worldOverlay);
            wpfMap1.ZoomLevelSet = GetCustomZoomLevelSet();

            //LayerOverlay loOnTheFlay = new LayerOverlay();
            //loOnTheFlay.Layers.Add(new Jpeg2000RasterLayer(@"..\..\App_Data\HTSL_1.jp2"));
            //loOnTheFlay.TileCache = new FileBitmapTileCache(@"D:\\CacheOnTheFly", "DecimalDegree");
            //wpfMap1.Overlays.Add(loOnTheFlay);

            LayerOverlay lo = new LayerOverlay();
            wpfMap1.Overlays.Add(lo);

            //Add an empty layer, then LayerOverlay will only load cache.
            lo.Layers.Add(new InMemoryFeatureLayer());

            //Directly load caches
            lo.TileCache = new FileBitmapTileCache(@"D:\\CacheFolder", "DecimalDegree");

            wpfMap1.CurrentExtent = new RectangleShape(77.68166, 12.9337, 77.69404, 12.91624);
            wpfMap1.Refresh();
        }

        private ZoomLevelSet GetCustomZoomLevelSet()
        {
            ZoomLevelSet zoomlevelSet = new ZoomLevelSet();

            foreach (var scale in CreateCustomScale(0))
            {
                zoomlevelSet.CustomZoomLevels.Add(new ZoomLevel(scale));
            }

            return zoomlevelSet;
        }

        private static double GetConfiguredZoomScale(double maxZoomScale, int configuredZoomLevelID)
        {
            double factorToDevide = Math.Pow(2, (configuredZoomLevelID));
            double configuredZoomScale = maxZoomScale / factorToDevide;
            return configuredZoomScale;
        }

        private static Collection<double> CreateCustomScale(int configuredZoomLevelID)
        {
            Collection<double> scalesToCache = new Collection<double>();
            const int totalNumberOfZoomLevelsTobeConfigured = 20 - 1;//Configured value starts from 0 till 19.Hence the constant is 19
            const string functionName = "CreateZoomLevels";
            try
            {
                ZoomLevelSet zoomLevelSet = new ZoomLevelSet();
                var upperZoomScale = zoomLevelSet.ZoomLevel20.Scale;
                var lowerZoomScale = 591658710.9;
                var configuredZoomScale = GetConfiguredZoomScale(lowerZoomScale, configuredZoomLevelID);

                var intialZoomScale = configuredZoomScale;
                int stepsNeededinBetweenScales = 0;
                double stepScale = 0;
                var availableZoomLevels = totalNumberOfZoomLevelsTobeConfigured - configuredZoomLevelID;
                //This many number of zoom levels are available for us from configuration
                if (configuredZoomLevelID != 0) //Recreate zoom levels
                {
                    if (availableZoomLevels != 0)
                    {
                        if (availableZoomLevels != 1)
                            stepsNeededinBetweenScales = (totalNumberOfZoomLevelsTobeConfigured - availableZoomLevels) /
                                                         (availableZoomLevels - 1);
                        //Only available availableZoomLevels - 1 levels
                        //are there for inserting step zoom level
                        else
                            stepsNeededinBetweenScales = totalNumberOfZoomLevelsTobeConfigured - 1;
                        // In this case we have only one step to insert all zoom levels.
                    }
                    scalesToCache.Add(configuredZoomScale);
                    for (int majorZoomLevel = 1; majorZoomLevel < availableZoomLevels; majorZoomLevel++)
                    {
                        for (int step = 1; step <= stepsNeededinBetweenScales; step++)
                        {
                            var lowerStepScale = intialZoomScale;
                            var upperStepScale = intialZoomScale / 2;
                            stepScale = (lowerStepScale - upperStepScale) / (stepsNeededinBetweenScales + 1);
                            // Since we are adding major zoom levels in upper loop
                            // adding 1 to stepsNeededinBetweenScales for calculation
                            if (scalesToCache.Count >= totalNumberOfZoomLevelsTobeConfigured - 1)
                                break; // If at all the count  goes more than 18 due to round off break the loop
                            scalesToCache.Add(lowerStepScale - stepScale * step);
                        }
                        intialZoomScale = intialZoomScale / 2;
                        scalesToCache.Add(intialZoomScale);
                    }
                    //Due to round off while calculting the number of stepsneeded inbetween, we will loose some zoom levels or get some zoom levels
                    if (scalesToCache.Count < totalNumberOfZoomLevelsTobeConfigured)
                    {
                        var deficitLevels = totalNumberOfZoomLevelsTobeConfigured - scalesToCache.Count;
                        var scaleToBeadded = (intialZoomScale - upperZoomScale) / (deficitLevels + 1);
                        // adding one to ensure that we can yet have one more last zoom level to be added
                        //We need to increase some steps because of round off while calculating number of steps needed in between
                        for (int step = 1; step <= deficitLevels; step++)
                            scalesToCache.Add((intialZoomScale - step * scaleToBeadded));
                    }
                    scalesToCache.Add(upperZoomScale);// This is the last scale for the map
                }
                else //If map is configured with 0 zoom level,add only default zoom levels
                {
                    for (int majorZoomLevel = configuredZoomLevelID;
                         majorZoomLevel <= totalNumberOfZoomLevelsTobeConfigured;
                         majorZoomLevel++)
                    {
                        intialZoomScale = intialZoomScale / 2;
                        scalesToCache.Add(intialZoomScale);
                    }
                }
                return scalesToCache;
            }
            catch (Exception ex)
            {
                throw;
            }
        }
    }
}